%
% Ocean  test problem. Demonstrates how to use subdomain deflation with idrs.m.
%
% The test problems are discretizations of Stommel's ocean circulation model. 
% Files are provided on four grids (spacing 1, 2, 3 or 4 degrees).
%
% Files have the following names and meanings:
%
%     stommel?.mtx:   the system matrix 
%     stommel?_b.mtx: the right-hand sides. Every file contains 12 
%                     rhs vectors, one for each month.
%     stommel?_P.mtx: map from system dofs to earth dofs, needed to
%                     set-up deflation vectors and plot the solution
%     bathymetry.mtx: contains waterdepth in each gridpoint (spacing 1 dg)
%
% The data files are supplied in Matrix Market format
%
% The discretisation is described in: 
%     M. B. van Gijzen, C. B. Vreugdenhil, and H. Oksuzoglu, 
%     The Finite Element Discretization for Stream-Function Problems 
%     on Multiply Connected Domains, 
%     J. Comp. Phys., 140, 1998, pp. 30-46. (copyright Academic Press)
% Please put a reference to this paper if you use thes test problems.
%
% This driver needs the following files: idrs.m, mv.m, subdomain_deflation.m 
%                                        mmread.m (matrix-market routine)
%
% The software is distributed without any warranty.
%
% Martin van Gijzen
% Copyright (c) June 2011
%
clear all;
close all;
deg = 4;
Title = 'Parameters';
sx = 0;
sy = 0;
s = 4;
maxit = 1000;
tol = 1e-8;
contin = 1;

c = ['b','g','r','c','m','y','b','g','r','c','m','y'];
exper = 0;

month = char('January ','February','March', 'April', 'May', 'June', 'July', ...
             'August','September','October','November','December');

while contin

   exper = exper + 1;
   prompt = {'Grid :','s:','Subdomains in x-direction','Subdomains in y-direction',...
             'Maximum number of iteratons','Tolerance'};
   defaults = {num2str(deg),num2str(s),num2str(sx),num2str(sy),num2str(maxit),num2str(tol)};
   lineNo=[1 25];
   params=inputdlg(prompt,Title,lineNo,defaults);
   if ( isempty(params) ) contin = 0; break; end;

%    Actual values:
   deg = str2num(char(params(1)));
   s  = str2num(char(params(2)));
   sx = str2num(char(params(3)));
   sy = str2num(char(params(4)));
   maxit = str2num(char(params(5)));
   tol = str2num(char(params(6)));

%    Read the matrices:
   matrix = ['stommel',num2str(deg),'.mtx'];
   rhs = ['stommel',num2str(deg),'_b.mtx'];
   map = ['stommel',num2str(deg),'_P.mtx'];
   [K, n, m, entries, rep, field, symm] = mmread(matrix);
   [rhs, n, nrhs, entries, rep, field, symm] = mmread(rhs);
   [P, n1, m1, entries, rep, field, symm] = mmread(map);
   [bathymetry, n1, m1, entries, rep, field, symm] = mmread('bathymetry.mtx');
%
%    Preconditioner:
   setup.type = 'nofill';
   [L, U] = ilu(K,setup);
%
%    Deflation
   num_defl = sx*sy;
   if ( num_defl > 0 ) 
      V = subdomain_deflation( n, P, deg, sx, sy );
      A.V = sparse(V);
      AV = K*V;
      A.AV = sparse(AV);
      H0 = V'*AV;
      A.H0 = H0;
   end 
%
%    Output: description of experiment:
   disp(['EXPERIMENT ',num2str(exper)]);
   disp('====================');
   disp('Problem definition.');
   disp(['Grid ', num2str(deg)]);
   disp(['Number of equations ', num2str(n)]);
   disp(' ');
   disp('Solver parameters.')
   disp(['s = ',num2str(s)]);
   disp(['maxit = ',num2str(maxit)]);
   disp(['tol = ',num2str(tol)]);
   disp('ILU preconditioning');
   if (num_defl == 0 )
      disp('No subdomain deflation');
   else 
      disp(['Subdomain deflation with ',num2str(sx),'x',num2str(sy),' subdomains']);
   end
%
%    Initialize figures
   disp(' ');
   scrsz = get(0,'ScreenSize');
   fig1 = figure('Position',[scrsz(1) 0.5*scrsz(4) 0.4*scrsz(3) 0.5*scrsz(4)]);
   title('Convergence');
   xlabel('Number of iterations');
   ylabel('Scaled residual norm');
   grid on;
   hold on;
   fig2 = figure('Position',[0.6*scrsz(3) 0.5*scrsz(4) 0.4*scrsz(3) 0.5*scrsz(4)]);
%
% Store information about the matrix
   A.name = 'mv';
   A.K = K;
   A.num_defl = num_defl;
%
% Loop over different months
   for irhs = 1:nrhs
%
%       Make new (deflated) rhs-vector:
      b = rhs(:,irhs);
      if ( num_defl > 0 )
         xc = V*(H0\(V'*b));
         b = b - AV*(H0\(V'*b));
      end
%
      t0 = cputime;
%
%        Solve the systems:
      [x, flag, relres, iter, resvec]=idrs(A,b,s,tol,maxit,L,U);
%
      time = cputime-t0;
%
      if ( num_defl > 0 )
         x = x - V*(H0\(V'*(K*x)));
         x = x + xc;
      end
%
%        Output:
      disp(['Results for ',month(irhs,:)]);
      disp(['Iterations: ',num2str(iter)]);
      disp(['CPU time: ',num2str(time),'s.']);
      resnrm = norm(K*x - rhs(:,irhs))/norm(rhs(:,irhs));
      disp(['Accuracy: ',num2str(resnrm)])
      disp(' ');
%
%        Plot convergence:
      figure(fig1);
      x_as = [0:1:iter];
      plot(x_as,log10(resvec/norm(b)),c(irhs));
      legend(month(1:irhs,:));

      drawnow;

%
%        Add extra line of gridpoints at North pole (the dirichlet boundary)
      x = [x; zeros(360/deg,1)];
%
%        Map solution on earth coordinates
      y = x(P);
      y = reshape(y,360/deg+1,180/deg);

%
%        Plot solution:
      figure(fig2);
      contour(0:deg:360,-(90-deg/2):deg:(90-deg/2),y',20)
      hold on;
      caxis([-1e5 5e4]);
      contour(0:360,-89.5:89.5,bathymetry',[0 0],'k')
      title(month(irhs,:));
      drawnow;
      hold off;
   end
end
