function V = subdomain_deflation( n, P, deg, sx, sy )
%
% Contruct subdomain deflation vectors for ocean test problems
%
% Called by ocean.m
%
% Input: n:   size of the system
%        P:   maps form 'system' degrees of freedom to earth
%             Note: the gridpoints on the North pole have been
%             removed from the system (streamfunction is prescribed)
%        deg: defines the grid spacing in degrees
%        sx:  number of subdomains in x-direction
%        sy:  number of subdomains in y-direction
%
% Output: V: deflation vectors
%
% Method: The earth is devided in sx*sy subdomains
%         Deflation vectors are made that are one in one
%         subdomain and zero in the others. These vectors are
%         then mapped on the system degrees of freedom.
%         Degrees of freedom that are in multiple subdomains
%         (continents, periodic boundaries) are averaged over
%         the subdomains. Their weights add up to one.
%
% The software is distributed without any warranty.
%
% Martin van Gijzen
% Copyright (c) June 2011
%

nx = 360/deg + 1;
x_stride = ceil(nx/sx);
ny = 180/deg;
y_stride = ceil(ny/sy);
y_start = 0;
V = zeros(n,sx*sy);
ik = 0;
for iy = 1:sy
   y_end = min(ny,y_start+y_stride);
   x_start = 0;
   for ix = 1:sx
      ik = ik +1;
      x_end = min(nx,x_start+x_stride);
      ve = zeros(nx,ny);
      ve(x_start+1:x_end,y_start+1:y_end) = 1;
      ve = reshape(ve,nx*ny,1);
%
% Note: ve contains ones in the subdomain in earth dofs
% Now it has to be mapped to system dofs
% the top-row of the earth dof is prescribed, therefore the (ny-1):
%
      for i = 1:nx*(ny-1)
         V(P(i),ik) = V(P(i),ik) + ve(i);
      end 
      x_start = x_end;
   end   
   y_start = y_end;
end 
%
% Make sure that the row-sum of the deflation vectors is 1:
%
for i = 1:n
   V(i,:) = V(i,:)./sum(V(i,:));
end

return
